#!/bin/bash

# Copyright (C) since 2014 NTT DATA Corporation
#
# This program is free software; you can redistribute it and/or
# Modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details

# bash configuration
SCRIPT_DIR=$(cd $(dirname $0);pwd)
PROG=`basename $0`

########################################
#  Local Variable
########################################

# script version
SCRIPT_VERSION="1.0.0"

# api version
MS_VERSION="2013-11-01"

# profile name
PROFILE=""

# initial lookup days
DAYS=30

# subscription id file
SUBSCRIPTION_ID_FILE="azure.id"

# certification file
CRT_FILE="azure.crt"

# private key file
PEM_FILE="azure.pem"

# cert root directory
CERT_ROOT_DIR=${SCRIPT_DIR}/cert

# log directory
LOG_ROOT_DIR=/var/log/azure/operationlog

# temporally directory
TMP_ROOT_DIR=/var/log/azure/operationlog/.tmp

# last modified date check file
LAST_MODIFIED_DATE_FILE="LastModifiedDate.txt"

# xml convert module
LOG_XML_PARSER="${SCRIPT_DIR}/HinemosAzureOperationLog.py"

# log file
LOG_FILE="AzureOperationLog.log"

########################################
# Function
########################################

#
# Message Logging
#
function Logging() {
	[ "x${QUIET}" != "xtrue" ] && echo "$1"
	/usr/bin/logger "${PROG} $1"
}

#
# show usage
#
function usage {
	echo "usage : ${PROG} -p profile -q [-d DAYS] [--version] [--help]"
	echo "options : "
	echo "  -p profile  set a profile (profile is same as directory name : /root/cloud/azure/cert/<profile>/)"
	echo "  -q          quiet mode"
	echo "  -d days     set period days of initial lookup [default:30]"
	echo "  --version   show version"
	echo "  --help      show usage"
}

########################################
# SHELL
########################################

# help check
for OPT in $@
do
	case $OPT in
		--version)
			echo ${SCRIPT_VERSION}
			exit 0
			;;
		--help)
			usage
			exit 0
			;;
	esac
done

# check parameters
while getopts p:d:q OPT
do
	case $OPT in
		p)
			PROFILE=${OPTARG}
			;;
		d)
			OPTION_D=true
			DAYS=${OPTARG}
			;;
		q)
			QUIET=true
			;;
		*)
			usage
			exit 1
			;;
	esac
done

shift $((OPTIND - 1))

if [ "x${PROFILE}" == "x" ];then
	echo "option (-p) is not specified."
	usage
	exit 1
fi

if [ "x${DAYS}" == "x" ] || [ `expr "${DAYS}" : '[0-9]*' > /dev/null` ]; then
	echo "option (-d) must be number."
	usage
	exit 1
fi

CERT_DIR=${CERT_ROOT_DIR}/${PROFILE}
[ -e ${CERT_DIR} ] || { Logging "certification directory \"${CERT_DIR}\" not found."; exit 1; }

[ -e ${CERT_DIR}/${SUBSCRIPTION_ID_FILE} ] || { Logging "subscription id file \"${CERT_DIR}/${SUBSCRIPTION_ID_FILE}\" not found"; exit 1; }
[ -e ${CERT_DIR}/${CRT_FILE} ] || { Logging "certification file \"${CERT_DIR}/${CRT_FILE}\" not found"; exit 1; }
[ -e ${CERT_DIR}/${PEM_FILE} ] || { Logging "private key file \"${CERT_DIR}/${PEM_FILE}\" not found"; exit 1; }
SUBSCRIPTION_ID=`cat ${CERT_DIR}/${SUBSCRIPTION_ID_FILE}`

LOG_DIR=${LOG_ROOT_DIR}/${PROFILE}
if [ ! -e ${LOG_DIR} ]; then
	Logging "creating log directory \"${LOG_DIR}\"."
	mkdir -p ${LOG_DIR} || { Logging "failed making directory \"${LOG_DIR}\""; exit 1; }
fi

TMP_DIR=${TMP_ROOT_DIR}/${PROFILE}
if [ ! -e ${TMP_DIR} ]; then
	mkdir -p ${TMP_DIR} || { Logging "failed making directory \"${TMP_DIR}\""; exit 1; }
fi

NOW=`TZ='UTC' date +"%Y-%m-%dT%H:%M:%SZ"`
YYYYMMDDHHMMSS=`TZ="UTC" date +"%Y%m%d%H%M%S"`
LAST_MODIFIED_DATE=`TZ="UTC" date -d "-${DAYS} day" +"%Y-%m-%dT%H:%M:%SZ"`
if [ -e ${LOG_DIR}/${LAST_MODIFIED_DATE_FILE} ]; then
	if [ "x${OPTION_D}" == "xtrue" ]; then
		Logging "file \"${LOG_DIR}/${LAST_MODIFIED_DATE_FILE}\" exists, option (-d) ignored."
	fi
	LAST_MODIFIED_DATE=`cat ${LOG_DIR}/${LAST_MODIFIED_DATE_FILE}`
fi

REQUEST_CNT=0
TOKEN="XXX"

Logging "downloading logs between \"${LAST_MODIFIED_DATE}\" and \"${NOW}\"..."
while [ "${TOKEN}" != "" ]; do
	# get operation log
	if [ "${REQUEST_CNT}" -eq 0 ]; then
		URL="https://management.core.windows.net/${SUBSCRIPTION_ID}/operations?StartTime=${LAST_MODIFIED_DATE}&EndTime=${NOW}"
		URL_MASKED="https://management.core.windows.net/XXXXXX/operations?StartTime=${LAST_MODIFIED_DATE}&EndTime=${NOW}"
	else
		URL="https://management.core.windows.net/${SUBSCRIPTION_ID}/operations?StartTime=${LAST_MODIFIED_DATE}&EndTime=${NOW}&ContinuationToken=${TOKEN}"
		URL_MASKED="https://management.core.windows.net/${SUBSCRIPTION_ID}/operations?StartTime=${LAST_MODIFIED_DATE}&EndTime=${NOW}&ContinuationToken=${TOKEN}"
	fi
	
	XML_FILE=${YYYYMMDDHHMMSS}_${REQUEST_CNT}.xml
	REQUEST_CNT=`expr ${REQUEST_CNT} + 1`
	
	curl -s --cert ${CERT_DIR}/${CRT_FILE} --key ${CERT_DIR}/${PEM_FILE} -H "Content-Type: text/html" -H "x-ms-version: ${MS_VERSION}" "${URL}" > ${TMP_DIR}/${XML_FILE} || \
		{ Logging "failed downloading from \"${URL_MASKED}\" to \"${TMP_DIR}/${XML_FILE}\""; exit 1; }
	
	TOKEN=`${LOG_XML_PARSER} -i ${TMP_DIR}/${XML_FILE} -o ${LOG_DIR}/${LOG_FILE}` || \
		{ Logging "failed parsing from \"${TMP_DIR}/${XML_FILE}\" to \"${LOG_DIR}/${LOG_FILE}\""; Logging "${TMP_DIR}/${XML_FILE} : `head -c 512 ${TMP_DIR}/${XML_FILE}`"; exit 1; }
done

echo ${NOW} > ${LOG_DIR}/${LAST_MODIFIED_DATE_FILE} || { Logging "failed save last timestamp to \"${LOG_DIR}/${LAST_MODIFIED_DATE_FILE}\""; exit 1; }

########################################
# Termination Processing
########################################

rm -rf ${TMP_DIR} || { Logging "failed delete temporally directory \"${TMP_DIR}\""; exit 1; }

Logging "updated files at ${NOW}"
Logging "   log file : ${LOG_DIR}/${LOG_FILE}"
Logging "   timestamp file : ${LOG_DIR}/${LAST_MODIFIED_DATE_FILE}"

exit 0
